#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::owner - unknown owner control module
#
package Security::owner;

use Security::find;
use Security::util::stat;
use Security::util::users;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &print() );

our (%gid, %nouids, %nogids);

sub init {
	print ( STDERR  "In Security::owner::init()\n") if ( $main::opts{'d'} );

	# Load up some tables for the find to run
	setgrent;
	while (my($nm, $pw, $gid) = getgrent) {
		if ( $nm ne '+' ) {

			$gid = 4294967294
               	if ($gid == -2 &&  $^O =~ m/aix/);

			$gid{$nm} = $gid{$gid} = $gid;
		}
	}
	endgrent;

	&Security::find::add_wanted (\&owner_wanted);
}

# Check for files that have no defined owner or group.
# From HP-UX System Security manual
sub owner_wanted {
	my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = 
		&Security::util::stat::get_stat($File::Find::name, 'lstat');

	# If the file has no known owner or group, add it to that list
     $nouids{$File::Find::name} = 1
		if ( defined $uid && !Security::util::users::check('uid',$uid));

     $nogids{$File::Find::name} = 1
		if ( defined $gid && !defined $gid{$gid} );
}

sub print {
	print ( STDERR  "In Security::owner::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking for files with no owner:\n", %nouids);
	&main::print_data ("\n\nchecking for files with no group:\n", %nogids);
}

1;

__END__

=head1 NAME

Security::owner - Look for files without an owner on the system

=head1 SYNOPSIS

Security::owner::init()

Security::owner::print()

=head1 DESCRIPTION

Look for files that have uid or gid values that currently do not exist on
the system.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Get the list of valid groups (via C<getgrent(3)>).

There is a hack to handle the case on AIX where gid == -2 comes back as
4294967294, even though the group file has -2.

Calls C<Security::find::add_wanted> to register the C<owner_wanted> sub routine.

=item C<print()>

Calls C<print_data ()> to print out the list of files with no owner.

Calls C<print_data ()> to print out the list of files with no group.

=back 

=head1 HISTORY

This idea of tracking missing owner or group information was suggested 
in the C<HP-UX System Security Manual>

=head1 SEE ALSO

L<Security::find>

L<Security::util::stat>

L<Security::util::users>

L<getgrent(3)>

=cut

